; /*
;  * Copyright (c) 1999 - 2003
;  * NetGroup, Politecnico di Torino (Italy)
;  * All rights reserved.
;  * 
;  * Redistribution And use in source And binary forms, With Or without 
;  * modification, are permitted provided that the following conditions 
;  * are met:
;  * 
;  * 1. Redistributions of source code must retain the above copyright 
;  * notice, this list of conditions And the following disclaimer.
;  * 2. Redistributions in binary form must reproduce the above copyright 
;  * notice, this list of conditions And the following disclaimer in the 
;  * documentation And/Or other materials provided With the distribution. 
;  * 3. Neither the name of the Politecnico di Torino nor the names of its 
;  * contributors may be used To endorse Or promote products derived from 
;  * this software without specific prior written permission. 
;  * 
;  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS And CONTRIBUTORS 
;  * "AS IS" And ANY EXPRESS Or IMPLIED WARRANTIES, INCLUDING, BUT Not 
;  * LIMITED To, THE IMPLIED WARRANTIES OF MERCHANTABILITY And FITNESS For 
;  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
;  * OWNER Or CONTRIBUTORS BE LIABLE For ANY DIRECT, INDIRECT, INCIDENTAL, 
;  * SPECIAL, EXEMPLARY, Or CONSEQUENTIAL DAMAGES (INCLUDING, BUT Not 
;  * LIMITED To, PROCUREMENT OF SUBSTITUTE GOODS Or SERVICES; LOSS OF USE, 
;  * Data, Or PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, Or TORT 
;  * (INCLUDING NEGLIGENCE Or OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
;  * OF THIS SOFTWARE, EVEN If ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;  * 
;  */

EnableExplicit
OpenConsole()
#AF_INET6 = 23

IncludePath "Include"
XIncludeFile "pcap.pb"

;/* 4 bytes IP address */
Structure ip_address
  byte1.b
  byte2.b
  byte3.b
  byte4.b
EndStructure

;/* IPv4 header */
Structure ip_header
  ver_ihl.b;        // Version (4 bits) + Internet header length (4 bits)
  tos.b;            // Type of service 
  tlen.w;           // Total length 
  identification.w; // Identification
  flags_fo.w;       // Flags (3 bits) + Fragment offset (13 bits)
  ttl.b;            // Time to live
  proto.b;          // Protocol
  crc.w;            // Header checksum
  saddr.ip_address; // Source address
  daddr.ip_address; // Destination address
  op_pad.l;         // Option + Padding
EndStructure

;/* UDP header*/
Structure udp_header
  sport.w;          // Source port
  dport.w;          // Destination port
  len.w;            // Datagram length
  crc.w;            // Checksum
EndStructure

;/* Prototype of the packet handler */
Declare packet_handler(*param.b, *header.pcap_pkthdr, *pkt_data.b)

Define *alldevs.pcap_if_t, *d.pcap_if_t, inum.l, i.l=0, *adhandle.pcap_t, errbuf.s{#PCAP_ERRBUF_SIZE}, netmask.l, fcode.bpf_program
Define packet_filter.s = "ip and udp"
Define *tmp_sockaddr_in.SOCKADDR_IN

;/* Retrieve the device list on the local machine */
If pcap_findalldevs_ex(#PCAP_SRC_IF_STRING, #Null, @*alldevs, @errbuf) = -1
  PrintN("Error in pcap_findalldevs: "+ errbuf)
  Input()
  End
EndIf
    
    ;/* Print the list */
*d = *alldevs
While *d
  
  i+1
  PrintN(Str(i)+": "+*d\name)
  
  If *d\description
    PrintN(*d\description);
  Else
    PrintN(" (No description available)");
  EndIf
  
  *d = *d\next
Wend  

If i = 0
  PrintN("No interfaces found! Make sure WinPcap is installed.")
  Input()
  End -1;
EndIf
    
Print("Enter the interface number (1-"+Str(i)+"): "): inum = Val(Input())
    
If (inum < 1) Or (inum > i)
  PrintN("Interface number out of range.")
  ;/* Free the device list */
  pcap_freealldevs(*alldevs)
  Input()
  End -1
EndIf
    
;/* Jump To the selected adapter */
*d = *alldevs
i = 0
While *d
  i+1
  If i = inum
    Break
  EndIf
  *d = *d\next
Wend
    
;/* Open the device */
*adhandle = pcap_open(*d\name, 65536, #PCAP_OPENFLAG_PROMISCUOUS, 1000, #Null, @errbuf)

If *adhandle = #Null
  PrintN("Unable to open the adapter. "+*d\name+" is not supported by WinPcap")
  ;/* Free the device list */
  pcap_freealldevs(*alldevs);
  Input()
  End -1;
EndIf
    
;/* Check the link layer. We support only Ethernet For simplicity. */
If pcap_datalink(*adhandle) <> #DLT_EN10MB
  PrintN("This program works only on Ethernet networks.")
  ;/* Free the device list */
  pcap_freealldevs(*alldevs)
  Input()
  End -1
EndIf
    
If *d\addresses <> #Null
  ;/* Retrieve the mask of the first address of the Interface */
  *tmp_sockaddr_in = *d\addresses\netmask
  netmask=*tmp_sockaddr_in\sin_addr
Else
  ;/* If the Interface is without addresses we suppose To be in a C class network */
  netmask=$ffffff
EndIf

;//compile the filter
If pcap_compile(*adhandle, @fcode, packet_filter, 1, netmask) < 0
  PrintN("Unable to compile the packet filter. Check the syntax.")
  ;/* Free the device list */
  pcap_freealldevs(*alldevs)
  Input()
  End -1
EndIf
    
;//set the filter
If pcap_setfilter(*adhandle, @fcode) < 0
  PrintN("Error setting the filter.")
  ;/* Free the device list */
  pcap_freealldevs(*alldevs)
  Input()
  End -1
EndIf
    
PrintN("listening on "+*d\description+"...")
    
;/* At this point, we don't need any more the device list. Free it */
pcap_freealldevs(*alldevs);
    
;/* start the capture */
pcap_loop(*adhandle, 0, @packet_handler(), #Null)
    
End 0

;/* Callback function invoked by libpcap For every incoming packet */
ProcedureC packet_handler(*param.b, *header.pcap_pkthdr, *pkt_data.b)
  Protected ltime, timestr.s{16}, *ih.ip_header, *uh.udp_header, ip_len.l, sport.w, dport.w

  ;/* convert the timestamp To readable format */
  If *header\ts
    timestr = FormatDate("%hh:%ii:%ss", *header\ts\tv_sec)
  EndIf

  ;/* print timestamp And length of the packet */
  Print(timestr+"."+RSet(Str(*header\ts\tv_usec),6,"0")+" len:"+Str(*header\len)+" ")

  ;/* retireve the position of the ip header */
  *ih = (*pkt_data + 14); //length of ethernet header

  ;/* retireve the position of the udp header */
  ip_len = (*ih\ver_ihl & $f) * 4
  *uh = (*ih + ip_len);

  ;/* convert from network byte order To host byte order */
  sport = ntohs_( *uh\sport );
  dport = ntohs_( *uh\dport );

  ;/* print ip addresses And udp ports */
  Print(StrU(*ih\saddr\byte1,#Byte)+"."+StrU(*ih\saddr\byte2,#Byte)+"."+StrU(*ih\saddr\byte3,#Byte)+"."+StrU(*ih\saddr\byte4,#Byte)+":"+StrU(sport,#Word)+" -> ")
  PrintN(StrU(*ih\daddr\byte1,#Byte)+"."+StrU(*ih\daddr\byte2,#Byte)+"."+StrU(*ih\daddr\byte3,#Byte)+"."+StrU(*ih\daddr\byte4,#Byte)+":"+StrU(dport,#Word))
EndProcedure
; IDE Options = PureBasic v4.00 (Windows - x86)
; CursorPosition = 74
; FirstLine = 50
; Folding = -